@@ -87,6 +87,7 @@ def shellquote(s: str):
 def install_shell_completion(
     parser: ArgumentParser,
     *commands: str,
+    extra_completion_script: Opt[str] = None,
     completion_options: Sequence[str] = DEFAULT_BASH_COMPLETE_OPTIONS,
     last_edit_time: Opt[Union[float, str, Path]] = None
 ):
@@ -105,11 +106,12 @@ def install_shell_completion(
                 outfile,
                 commands=commands,
                 completion_options=completion_options,
+                extra_completion_script=extra_completion_script,
             )
 
     custom_source_command = BASH_SOURCE_TEMPLATE.format(custom_file, custom_file)
     _ensure_lines([custom_source_command], completions_file_abs)
-    print("eval the following in your shell to update completions:")
+    print("eval the following in your shell to update completions:", file=sys.stderr)
     print(custom_source_command, file=sys.stdout)
 
 
@@ -163,6 +165,11 @@ class FixedShellFunctionComplete(Complete):
         self.args = args
 
 
+class CompleteFromStdout(FixedShellFunctionComplete):
+    """Complete from a shell command that prints lines to stdout"""
+    _shell_func_name = '_bourbaki_complete_from_stdout'
+
+
 class _BashCompletionCompleters:
     """Constuct completers from the `bash_completion` bash module, which are highly developed and heavily tested.
     This uses simple .attribute access using '_'-stripped bash_completion shell function names."""
@@ -331,6 +338,7 @@ def write_bash_completion_for_parser(
     file: IO[str],
     commands: Sequence[str],
     completion_options: Opt[Union[str, Sequence[str]]] = None,
+    extra_completion_script: Opt[str] = None,
     shebang: Opt[str] = BASH_SHEBANG,
 ):
     if isinstance(commands, str):
@@ -368,6 +376,10 @@ def write_bash_completion_for_parser(
         line = "complete {}-F {} {}".format(optionstr, completion_funcname, cmd)
         print_(line)
 
+    if extra_completion_script:
+        print_()
+        print_(extra_completion_script)
+
     print_()
 
 
@@ -536,13 +548,30 @@ def _ensure_lines(lines, textfile: str):
 
 
 def _install_shell_completion_helpers(completions_helpers_file):
-    source = pkgutil.get_data('bourbaki.application.completion', BASH_COMPLETION_HELPERS_FILENAME).strip()
+    source = pkgutil.get_data(
+        "bourbaki.application.completion", BASH_COMPLETION_HELPERS_FILENAME
+    ).strip()
     completions_helpers_file_absolute = os.path.expanduser(completions_helpers_file)
 
-    with open(completions_helpers_file_absolute, "rb+") as outfile:
-        old_source = outfile.read().strip()
-        if source != old_source:
-            print("REINSTALLING BASH COMPLETION HELPERS AT {}".format(completions_helpers_file_absolute), file=sys.stderr)
-            outfile.seek(0)
-            outfile.truncate()
+    if not os.path.exists(completions_helpers_file_absolute):
+        with open(completions_helpers_file_absolute, "wb") as outfile:
+            print(
+                "INSTALLING BASH COMPLETION HELPERS AT {}".format(
+                    completions_helpers_file_absolute
+                ),
+                file=sys.stderr,
+            )
             outfile.write(source)
+    else:
+        with open(completions_helpers_file_absolute, "rb+") as outfile:
+            old_source = outfile.read().strip()
+            if source != old_source:
+                print(
+                    "REINSTALLING BASH COMPLETION HELPERS AT {}".format(
+                        completions_helpers_file_absolute
+                    ),
+                    file=sys.stderr,
+                )
+                outfile.seek(0)
+                outfile.truncate()
+                outfile.write(source)
